@@ -1,11 +1,3 @@
-#from mstrio.connection import Connection
-#from mstrio.connection import get_connection
-#from mstrio.utils import parser
-#from mstrio.project_objects.datasets import SuperCube
-#from mstrio.api import reports
-#from mstrio.api import projects
-#from mstrio.api import objects as api_obj
-#from mstrio.project_objects import report
 from mstr_robotics.mstr_classes import rep
 from mstr_robotics.mstr_classes import mstr_global
 from mstr_robotics.mstr_classes import get_conn
@@ -18,7 +10,7 @@ class open_conn():
 
     def login(self,base_url,project_id=None,*args,**kwargs):
         return get_conn(self,base_url=base_url, project_id=project_id,*args,**kwargs)
-
+"""
 class prepare_mig_obj():
 
     def set_env_var(self, conn_det=None, change_log_report=None, change_query=None):
@@ -26,12 +18,11 @@ class prepare_mig_obj():
         self.i_datetime = datetime
         self.conn_det = conn_det
         self.conn = self.glob.get_conn(conn_det=conn_det, project_id=change_log_report["chg_log_rep_proj_id"])
-
+"""
 class cubes:
     def __init__(self):
         self.glob = mstr_global
 
-
     def up_load_to_cube(self,conn,cube_upload_param):
 
         self.glob.cube_upload(self,conn=conn
@@ -49,47 +40,38 @@ class bld_short_cuts:
         self.rep = rep()
         self.glob = mstr_global()
 
-    def bld_change_log_shortcut_df(self,conn,short_cut_df,change_log_df):
-        #user_id,  object_id,object_name,object_type
-        short_cut_df["NEW_OBJECT_NAME"]=""
-        short_cut_df["DESC_STR"] = ""
-        for index, obj in short_cut_df.iterrows():
-            #short_cut_obj=self.glob.get_short_cut_obj(conn=conn, project_id="project_id",object_id=obj["OBJECT_ID"], type=18)
-            #builds the Strings for shortCut Name & Descriptions
-            #name user_login__object_name__object_guid
-            short_cut_df.at[index,"NEW_OBJECT_NAME"]  = obj["USER_LOGIN"]+ "__" + obj["OBJECT_NAME"] + "__" + obj["OBJECT_ID"]
-            short_cut_df.at[index, "OBJECT_DESC"]=self._bld_desc_str(change_log_df.loc[change_log_df["OBJECT_ID"] == obj["OBJECT_ID"]])
-        return short_cut_df
-
     def bld_short_cut(self,conn,project_id,folder_id,object_id,
                       object_type,new_name,object_desc):
         #In Version 11 update 7, it's not possible to define a free name and description
         # when creating a short cut. Thus I create the shurt cut first and rename it later
         short_cut_obj_rep = self.glob._cr_short_cut(conn=conn,object_id=object_id,object_type=object_type,folder_id=folder_id)
-        #opens an instance of the created short cut object
-        short_cut_obj=self.glob.get_short_cut_obj(conn=conn, project_id=project_id,
-                                                  object_id=short_cut_obj_rep.json()["id"],
-                                                  type=18)
-        self.glob._rename_object(object=short_cut_obj, new_name=new_name, desc_str=object_desc)
+        self.glob._rename_object(conn=conn,project_id=project_id,object_id=short_cut_obj_rep.json()["id"],object_type=18,new_name=new_name, desc_str=object_desc)
 
-    def run_short_cut_build(self, conn,project_id,short_cut_df,folder_id,object_id_col,object_name_col,object_desc_col,object_type_col):
-        #requiers a pandas data frame, based on the output of the change log report
-        #here we count the number of changes at level of user & object
+    def run_short_cut_build(self, conn,project_id,short_cut_df,folder_id,df_key_cols_d=None):
+        #requiers a pandas data frame with 4 colmuns
+        # object_id, object_type, object_name, object_desc
+        if not df_key_cols_d:
+            df_key_cols_d={"object_id":"OBJECT_ID",
+                           "object_type":"OBJECT_TYPE",
+                           "object_name":"OBJECT_NAME",
+                           "object_desc":"OBJECT_DESC"}
 
+        #before adding new short cuts to the shortcut folder, all shortcuts gonna be deleted
         self.clean_shortcut_folder(conn=conn,project_id=project_id,folder_id=folder_id)
         short_cut_folder_j = f'{{"folderId": "{folder_id}"}}'
         #iterrating throuh all the objects in the list
         short_cut_df.reset_index()
         for index, obj in short_cut_df.iterrows():
             #short_cut_url = f'{conn.base_url}/objects/{obj[2]}/type/{str(obj[4])}/shortcuts'
-            if obj[4]!=str(18):
+            if obj[df_key_cols_d["object_type"]]!=str(18):
                 self.bld_short_cut(conn=conn
                                    ,project_id=project_id
                                    ,folder_id=folder_id
-                                   ,object_id=obj[object_id_col]
-                                   ,object_type=obj[object_type_col]
-                                   ,new_name=obj[object_name_col]
-                                   ,object_desc=self.str._get_first_x_chars(str_=obj[object_desc_col],i=249)
+                                   ,object_id=obj[df_key_cols_d["object_id"]]
+                                   ,object_type=obj[df_key_cols_d["object_type"]]
+                                   ,new_name=obj[df_key_cols_d["object_name"]]
+                                   #max lentgh of descriptions in MSTR
+                                   ,object_desc=self.str._get_first_x_chars(str_=obj[df_key_cols_d["object_desc"]],i=249)
                                    )
             else:
                 print(f'Obect: {obj[2]} is a shortcut. MSTR does not allow short cuts o short cuts,'
@@ -97,7 +79,6 @@ class bld_short_cuts:
 
     #@logger
     def clean_shortcut_folder(self,conn,project_id,folder_id):
-        #before adding new short cuts to the shortcut folder, all shortcuts gonna be deleted
         existing_obj_l = self.glob.get_folder_obj_l(conn=conn, project_id=project_id, folder_id=folder_id)
         obj_l = self._get_col_from_obj_l(dict_l=existing_obj_l, key_col="id")
         self.glob._delete_object(conn=conn,project_id=project_id,existing_obj_l=existing_obj_l)
@@ -109,21 +90,13 @@ class bld_short_cuts:
             key_val_l.append(o[key_col])
         return key_val_l
 
-    #@logger
-    def _bld_desc_str(self, all_changes=None):
-        desc_str = ""
-        all_changes_sum = all_changes.groupby(["USER_ID", "USER_LOGIN", "OBJECT_ID", "OBJECT_NAME", "OBJECT_TYPE"])[
-            "TRANSACTION_ID"].agg("count").reset_index()
-        for index, s in all_changes_sum.iterrows():
-            desc_str += s["USER_LOGIN"] + " (" + str(s["TRANSACTION_ID"]) + "), "
-        return desc_str
-
 class get_change_log:
 
     def __init__(self):
         self.str= str_func()
         self.rep = rep()
         self.glob = mstr_global()
+        self.run_shortcut=bld_short_cuts()
 
     def set_md_rep_params(self,conn,change_log_report,change_query):
         self.conn=conn
@@ -133,13 +106,38 @@ class get_change_log:
         self.chg_log_to_date_prompt_id = change_log_report["chg_log_to_date_prompt_id"]
         self.chg_log_proj_prompt_id = change_log_report["chg_log_proj_prompt_id"]
         self.chg_log_obj_prompt_id = change_log_report["chg_log_obj_prompt_id"]
-
+        """
         self.short_cut_proj_id = change_query["short_cut_proj_id"]
         self.chg_log_proj_name = "MicroStrategy Tutorial"
         self.chg_log_from_date = change_query["chg_log_from_date"]
         self.chg_log_to_date = change_query["chg_log_to_date"]
         #self.chg_log_dep_bool = change_query["chg_log_dep_bool"]
-        self.short_cut_folder_id=None
+        """
+        #self.short_cut_folder_id=None
+
+    #@logger
+    def _bld_desc_str(self, all_changes=None):
+        desc_str = ""
+        all_changes_sum = all_changes.groupby(["USER_ID", "USER_LOGIN", "OBJECT_ID", "OBJECT_NAME", "OBJECT_TYPE"])[
+            "TRANSACTION_ID"].agg("count").reset_index()
+        for index, s in all_changes_sum.iterrows():
+            desc_str += s["USER_LOGIN"] + " (" + str(s["TRANSACTION_ID"]) + "), "
+        return desc_str
+
+    def bld_change_log_shortcut_df(self,conn,change_log_df):
+
+        short_cut_df = change_log_df.groupby(["USER_ID", "USER_LOGIN", "OBJECT_ID", "OBJECT_NAME", "OBJECT_TYPE"])[
+            "TRANSACTION_ID"].agg("count").reset_index()
+
+        short_cut_df["NEW_OBJECT_NAME"]=""
+        short_cut_df["DESC_STR"] = ""
+        for index, obj in short_cut_df.iterrows():
+            #short_cut_obj=self.glob.get_short_cut_obj(conn=conn, project_id="project_id",object_id=obj["OBJECT_ID"], type=18)
+            #builds the Strings for shortCut Name & Descriptions
+            #name user_login__object_name__object_guid
+            short_cut_df.at[index,"NEW_OBJECT_NAME"]  = obj["USER_LOGIN"]+ "__" + obj["OBJECT_NAME"] + "__" + obj["OBJECT_ID"]
+            short_cut_df.at[index, "NEW_OBJECT_DESC"]=self._bld_desc_str(change_log_df.loc[change_log_df["OBJECT_ID"] == obj["OBJECT_ID"]])
+        return short_cut_df
 
     def parse_chglog_rep_cols(self,rep_def):
         cols=[]
@@ -168,11 +166,12 @@ class get_change_log:
 
         return comp_res
 
-    def chg_rep_to_df(self,prompt_answ):
-        instance_id = self.rep.open_Instance(conn=self.conn, project_id=self.short_cut_proj_id,
+    def chg_rep_to_df(self,conn,prompt_answ):
+
+        instance_id = self.rep.open_Instance(conn=conn, project_id=self.chg_log_rep_proj_id,
                                              report_id=self.chg_log_report_id)
 
-        self.rep.set_inst_prompt_ans(conn=self.conn, project_id=self.short_cut_proj_id,
+        self.rep.set_inst_prompt_ans(conn=conn,
                                      report_id=self.chg_log_report_id, instance_id=instance_id,
                                      prompt_answ=prompt_answ)
 
@@ -180,7 +179,7 @@ class get_change_log:
         cols=self.parse_chglog_rep_cols(rep_def)
         cols.append("path")
 
-        report_dict = self.rep.report_dict(conn=self.conn, project_id=self.short_cut_proj_id,
+        report_dict = self.rep.report_dict(conn=self.conn,
                                            report_id=self.chg_log_report_id, instance_id=instance_id)
 
         report_list = self.read_obj(report_dict)
@@ -188,50 +187,23 @@ class get_change_log:
 
         return report_df
 
-    def obj_id_from_df_l(self, report_df,obj_col_ind,type_col_ind):
-        obj_l=[]
-        for index, obj in report_df.iterrows():
-            obj_l.append(obj[obj_col_ind] + ";"+obj[type_col_ind])
-        return dict.keys(dict.fromkeys(obj_l))
-
-    def bld_prompt_obj_where_list(self,obj_l):
-        where_str=""
-        for o in obj_l:
-            where_str+="'" + self.glob.bld_mstr_obj_md_guid(o) +"',"
-
-        return self.str._rem_last_char(where_str,1)
-
-    def get_obj_l_from_search_l(self,search_l):
-        obj_l=[]
-        for o in search_l:
-            obj_l.append(o["object_id"])
-
-        return obj_l
-
-    def get_change_log_df_from_list(self,chg_log_rep_proj_id,obj_l ):
-        obj_where_clause_str = self.bld_prompt_obj_where_list(obj_l=obj_l)
-        report_df = self.chg_rep_to_df(self._build_val_answ(
-                            chg_log_rep_proj_id=self.chg_log_rep_proj_id,
-                            obj_where_clause_str=obj_where_clause_str))
-        return report_df
-
-    def get_mig_obj_logs(self ):
-        report_df=self.chg_rep_to_df(self._build_val_answ(
-                        chg_log_rep_proj_id=self.chg_log_rep_proj_id,
-                        chg_log_from_date =self.chg_log_from_date,
-                        chg_log_to_date=self.chg_log_to_date))
-        return report_df
-        """ 
-        if self.chg_log_dep_bool==True:
-            obj_id_l =self.obj_id_from_df_l(report_df,obj_col_ind=7,type_col_ind=10)
-            depn_obj_dict_l=self.glob.used_by_obj_rec_l(conn=self.conn, project_id=self.chg_log_rep_proj_id, obj_l=obj_id_l)
-            #depn_obj_dict_l=dict.keys(dict.fromkeys(depn_obj_dict_l))
-            obj_where_clause_str = self.bld_prompt_obj_where_list(depn_obj_dict_l)
-            #self.bld_prompt_obj_where_list(report_df)
-            report_df=self.chg_rep_to_df(self._build_val_answ(
-                                                         chg_log_rep_proj_id=self.chg_log_rep_proj_id,
-                                                         obj_where_clause_str=obj_where_clause_str))
-        """
+    def get_mig_obj_logs(self,conn,proect_id,from_date,to_date):
+        #get raw data from change logs
+        return self.chg_rep_to_df(conn, prompt_answ =self._build_val_answ(
+                        chg_log_rep_proj_id=proect_id,
+                        chg_log_from_date =from_date,
+                        chg_log_to_date=to_date))
+        #group by developer /user_id and object
+        #generate strings for name and description of the short cuts
+        short_cut_df = self.bld_change_log_shortcut_df(conn=self.conn, change_log_df=chg_log_df)
+        #define cols for short cut run
+        df_key_cols_d = {"object_id": "OBJECT_ID",
+                         "object_type": "OBJECT_TYPE",
+                         "object_name": "NEW_OBJECT_NAME",
+                         "object_desc": "NEW_OBJECT_DESC"}
+        self.run_shortcut.run_short_cut_build(conn=self.conn, project_id=self.short_cut_proj_id,
+                                          short_cut_df=short_cut_df, folder_id=self.short_cut_folder_id,
+                                          df_key_cols_d=df_key_cols_d)
 
     def read_obj(self, report_dict=None):
         project_id_ind=1
@@ -252,9 +224,9 @@ class get_change_log:
             # obj_l=self.bld_obj_l(obj_d=obj_d.json(),proj_g=self.glob.bld_mstr_obj_guid(o[project_id_ind]))
 
             if obj_d.status_code == 200 \
-                    and self.glob.bld_mstr_obj_guid(o[obj_g_ind]) != self.short_cut_folder_id:
-                fold_path = str(self.glob.bld_obj_path(fld_d=obj_d.json()["ancestors"], proj_id=self.short_cut_proj_id,
-                                                       proj_name=self.glob.get_project_name(conn=self.conn,project_id=self.chg_log_rep_proj_id)))
+                    and self.glob.bld_mstr_obj_guid(o[obj_typ_ind]) != 18: # excluding short cuts
+                fold_path = str(self.glob.bld_obj_path(fld_d=obj_d.json()["ancestors"], proj_id=self.glob.bld_mstr_obj_guid(o[project_id_ind]),
+                                                       proj_name=self.glob.get_project_name(conn=self.conn,project_id=self.glob.bld_mstr_obj_guid(o[project_id_ind]))))
 
                 if fold_path != "\\Managed Objects" \
                         and self.str._get_first_x_chars(str_=fold_path, i=15) != "\System Objects" \
@@ -304,6 +276,33 @@ class get_change_log:
             prompt_ans = f'{{"prompts":[{prompt_ans}]}}'
         return prompt_ans
 
+    def bld_prompt_obj_where_list(self,obj_l):
+        where_str=""
+        for o in obj_l:
+            where_str+="'" + self.glob.bld_mstr_obj_md_guid(o) +"',"
+
+        return self.str._rem_last_char(where_str,1)
+
+    def obj_id_from_df_l(self, report_df,obj_col_ind,type_col_ind):
+        obj_l=[]
+        for index, obj in report_df.iterrows():
+            obj_l.append(obj[obj_col_ind] + ";"+obj[type_col_ind])
+        return dict.keys(dict.fromkeys(obj_l))
+
+    def get_change_log_df_from_list(self,chg_log_rep_proj_id,obj_l ):
+        obj_where_clause_str = self.bld_prompt_obj_where_list(obj_l=obj_l)
+        report_df = self.chg_rep_to_df(self._build_val_answ(
+                            chg_log_rep_proj_id=self.chg_log_rep_proj_id,
+                            obj_where_clause_str=obj_where_clause_str))
+        return report_df
+
+    def get_obj_l_from_search_l(self,search_l):
+        obj_l=[]
+        for o in search_l:
+            obj_l.append(o["object_id"])
+
+        return obj_l
+
 class searches:
 
     def __init__(self):
